import { useEffect } from 'react';
import { useSelector, useDispatch } from 'react-redux'
import AppBar from './AppBar';
import RecordList from './RecordList';
import PlayerBar from './PlayerBar';
import styles from './MainPage.module.css';
import store from './business/store';
import yzs from "./business/request.js";
import { setList, setCurrentLyric, setCurrentBlob, setCurrentWaveData } from "./business/recorderSlice.js"
import { CssBaseline, Box } from '@mui/material';
import RecordLyrics from './RecordLyrics';
import { createTheme, ThemeProvider } from '@mui/material/styles';

const theme = createTheme({
    status: {
        danger: '#e53e3e',
    },
    palette: {
        primary: {
            main: '#FF595A',
            darker: '#FF595A',
        },
        neutral: {
            main: '#64748B',
            contrastText: '#fff',
        },
        black: {
            main: "#222222",
        }
    },
});


function fetchRecord(accessToken, record) {
    yzs.download(accessToken, record.transResultUrl).then(
        blob => blob.text()
    ).then(text => {
        console.log("type", record.type, text);
        let payload = record.type === 1 ? JSON.parse(text) : text;
        store.dispatch(setCurrentLyric(payload));
    });
    yzs.download(accessToken, record.audioUrl).then(blob => {
        store.dispatch(setCurrentBlob(URL.createObjectURL(blob)));
        blob.arrayBuffer().then(arrayBuffer => {
            let context = new (window.AudioContext || window.webkitAudioContext)();
            context.decodeAudioData(arrayBuffer).then(audioBuffer => {
                let interval = audioBuffer.sampleRate / 1000 * 100;
                let waveData = audioBuffer.getChannelData(0);
                let wave = [];
                let amplitude = 0;
                for (let i = 0; i < waveData.length; i++) {
                    amplitude += Math.abs(waveData[i]);
                    if (i % interval == 0) {
                        wave.push(Math.floor(amplitude * 500 / interval));
                        amplitude = 0;
                    }
                }
                store.dispatch(setCurrentWaveData(wave));
            })
        });
    });
}

export default function () {
    const dispatch = useDispatch()
    const accessToken = useSelector(state => state.user.accessToken);
    const passportId = useSelector(state => state.user.passportId);
    const currentTime = useSelector(state => state.recorder.currentTime);
    useEffect(() => {
        yzs.get_record_list(accessToken, passportId).then(list => {
            dispatch(setList(list.result));
            if (list.result.length > 0) {
                fetchRecord(accessToken, list.result.at(0));
            }
        }).catch(error => {
            console.log("get list failed", error);
        });
    }, [accessToken, passportId]);
    return <div className={styles.mainBody}>
        <ThemeProvider theme={theme}>
            <CssBaseline />
            <AppBar />
            <RecordList fetchRecord={fetchRecord} />
            <Box
                component="main"
                sx={{
                    width: `calc(100% - 296px)`,
                    ml: `272px`,
                    mr: `24px`,
                }}
            >
                <PlayerBar currentTime={currentTime} />
                <RecordLyrics currentTime={currentTime} />
            </Box>
        </ThemeProvider>
    </div >
}